@@ -11,7 +11,7 @@ import re
 
 from http import HTTPStatus
 from pathlib import Path
-from ssl import SSLContext
+from ssl import create_default_context, SSLContext
 from typing import Any, List, Union
 from urllib.parse import urlparse
 
@@ -77,6 +77,7 @@ class DockerRegistryClientAsync:
         """
         Args:
             credentials_store: Path to the docker registry credentials store.
+            ssl: SSL context.
             token_based_endpoints: List of token-based endpoints
         """
         if not credentials_store:
@@ -86,6 +87,12 @@ class DockerRegistryClientAsync:
                     DockerRegistryClientAsync.DEFAULT_CREDENTIALS_STORE,
                 )
             )
+        if not ssl:
+            cacerts = os.environ.get("DRCA_CACERTS", None)
+            if cacerts:
+                LOGGER.debug("Using cacerts: %s", cacerts)
+                ssl = create_default_context(cafile=str(cacerts))
+
         if not token_based_endpoints:
             token_based_endpoints = os.environ.get(
                 "DRCA_TOKEN_BASED_ENDPOINTS",
@@ -125,7 +132,7 @@ class DockerRegistryClientAsync:
             await self.client_session.close()
 
     async def _get_auth_token(
-        self, *, credentials: str, endpoint: str, scope: str
+        self, *, credentials: str = None, endpoint: str, scope: str
     ) -> str:
         """
         Retrieves the registry auth token for a given scope.
@@ -146,7 +153,9 @@ class DockerRegistryClientAsync:
         # https://github.com/docker/distribution/blob/master/docs/spec/auth/token.md
         if scope not in self.tokens[endpoint]:
             # Test using HTTP basic authentication to retrieve the www-authenticate response header ...
-            headers = {"Authorization": f"Basic {credentials}"}
+            headers = {}
+            if credentials:
+                headers["Authorization"] = f"Basic {credentials}"
 
             client_session = await self._get_client_session()
 
@@ -234,14 +243,13 @@ class DockerRegistryClientAsync:
 
         endpoint = image_name.resolve_endpoint()
         credentials = await self._get_credentials(endpoint)
-        if credentials:
-            if endpoint in self.token_based_endpoints:
-                token = await self._get_auth_token(
-                    credentials=credentials, endpoint=endpoint, scope=scope
-                )
-                headers["Authorization"] = f"Bearer {token}"
-            else:
-                headers["Authorization"] = f"Basic {credentials}"
+        if endpoint in self.token_based_endpoints:
+            token = await self._get_auth_token(
+                credentials=credentials, endpoint=endpoint, scope=scope
+            )
+            headers["Authorization"] = f"Bearer {token}"
+        elif credentials:
+            headers["Authorization"] = f"Basic {credentials}"
 
         return headers
 
@@ -638,10 +646,9 @@ class DockerRegistryClientAsync:
         Returns:
             The underlying client response.
         """
-        identifier = ""
         if image_name.digest:
             identifier = image_name.resolve_digest()
-        elif image_name.tag:
+        else:
             identifier = image_name.resolve_tag()
         if accept is None:
             accept = (
@@ -740,6 +747,32 @@ class DockerRegistryClientAsync:
         client_session = await self._get_client_session()
         return await client_session.get(headers=headers, url=url, **kwargs)
 
+    async def get_tag_list(
+        self, image_name: ImageName, **kwargs
+    ) -> DockerRegistryClientAsyncGetTags:
+        """
+        Fetch the tags under the repository identified by name.
+
+        Args:
+            image_name: The image name.
+        Keyword Args:
+            protocol: Protocol to use when connecting to the endpoint.
+
+        Returns:
+            dict:
+                client_response: The underlying client response.
+                tags: The corresponding list of image tags.
+        """
+        client_response = await self._get_tags(
+            image_name, raise_for_status=True, **kwargs
+        )
+        tags = await client_response.json()
+        tags = [
+            ImageName(image_name.image, endpoint=image_name.endpoint, tag=tag)
+            for tag in tags["tags"]
+        ]
+        return {"client_response": client_response, "tags": tags}
+
     async def get_tags(
         self, image_name: ImageName, **kwargs
     ) -> DockerRegistryClientAsyncGetTags:
@@ -879,10 +912,9 @@ class DockerRegistryClientAsync:
         Returns:
             The underlying client response.
         """
-        identifier = ""
         if image_name.digest:
             identifier = image_name.resolve_digest()
-        elif image_name.tag:
+        else:
             identifier = image_name.resolve_tag()
         protocol = kwargs.pop("protocol", DockerRegistryClientAsync.DEFAULT_PROTOCOL)
 
@@ -1263,10 +1295,9 @@ class DockerRegistryClientAsync:
         Returns:
             The underlying client response.
         """
-        identifier = ""
         if image_name.digest:
             identifier = image_name.resolve_digest()
-        elif image_name.tag:
+        else:
             identifier = image_name.resolve_tag()
         protocol = kwargs.pop("protocol", DockerRegistryClientAsync.DEFAULT_PROTOCOL)
 
